/*
 * Copyright 
 *   2007 axYus - www.axyus.com
 *   2007 N.Le Corre - nicolas.lecorre@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.cp.xemelios.controls.paye;

import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.xemelios.controls.AbstractUnitControl;
import fr.gouv.finances.cp.xemelios.controls.Anomalie;
import fr.gouv.finances.cp.xemelios.controls.IdGenerator;
import fr.gouv.finances.cp.xemelios.controls.Node;
import java.util.TreeSet;

public class PAY04 extends AbstractUnitControl {
	private final static Logger logger = Logger.getLogger(PAY04.class);
    public static final transient String CTRL_ID = PAY04.class.getSimpleName();
	private Vector<Anomalie> anos = new Vector<Anomalie>();
	private String docId = null;

	
	private static final String ON_RETURN_ANOMALIES = "DocumentPaye/";
	private static final String CHEMIN_DOCUMENT = "DocumentPaye/";
	private String refNodeId = "";


	private static final String CHEMIN_CODE_RUBRIQUE_EVENEMENT = "DocumentPaye/DonneesIndiv/PayeIndivMensuel/Evenement/Code/";
	private static final String CHEMIN_CODE_NOMENCLATURE_RUBRIQUE_EVENEMENT = "DocumentPaye/Nomenclatures/NomenclatureEvtPaye/Correspondance/Code/";

	private static final String REPLACE_LISTE = "##MESSAGE_MULTIPLE_DANS_LE_CODE##";
	private TreeSet<String> codesEvt = new TreeSet<String>();
	private Vector<String> codesEvtNomencl = new Vector<String>();
	
	public void startDocument() throws SAXException {
		logger.info("In "+CTRL_ID);
	}
	
	public void setDocId(String docID) {
		this.docId = docID;		
	}

	public void startElement(String uri, String localName, String qName, Attributes atts, String xpath) throws SAXException {
		if (xpath.endsWith(CHEMIN_DOCUMENT)) {
			refNodeId = atts.getValue("ano:node-id");
		} else if (xpath.endsWith(CHEMIN_CODE_NOMENCLATURE_RUBRIQUE_EVENEMENT)) {
			codesEvtNomencl.add(atts.getValue("V"));
		} else if (xpath.endsWith(CHEMIN_CODE_RUBRIQUE_EVENEMENT)) {
			codesEvt.add(atts.getValue("V"));
		}
	}

	public Vector<Anomalie> endElement(String uri, String localName, String qName, String xpath) throws SAXException {
		if (xpath.endsWith(ON_RETURN_ANOMALIES)) {
			// TEST
			String codesUtiliseNonDeclares = "<ul>";
			String codesDeclaresNonUtilises = "<ul>";
			for (String s:codesEvt) {
				if (!codesEvtNomencl.contains(s)) {
					codesUtiliseNonDeclares += "<li>"+s+"</li>";
				}
			}
			if ("<ul>".equals(codesUtiliseNonDeclares)) codesUtiliseNonDeclares = "";
			else codesUtiliseNonDeclares += "</ul>";
			for (String s:codesEvtNomencl) {
				if (!codesEvt.contains(s)) {
					codesDeclaresNonUtilises += "<li>"+s+"</li>";
				}
			}
			if ("<ul>".equals(codesDeclaresNonUtilises)) codesDeclaresNonUtilises = "";
			else codesDeclaresNonUtilises += "</ul>";
			
			// ANOMALIE
			String msg = "";
			if (!"".equals(codesUtiliseNonDeclares)) {
				msg = "Les codes vnements suivants sont utiliss mais non dclars dans la nomenclature : "+codesUtiliseNonDeclares;
				//String path = ""; // path est vide de faon  ne pas avoir de lien sous l'anomalie. 
				String anoId = IdGenerator.nextId();
				String libelleLien = "";
				Hashtable<String, Object> xslParams = new Hashtable<String, Object>();
				xslParams.put("elementId", "Paye");
				xslParams.put("browser-destination", "internal");
				xslParams.put("anoId", anoId);
				Anomalie ano = new Anomalie(anoId,
											getDocumentModel().getControlById(CTRL_ID).getId(),
											getDocumentModel().getControlById(CTRL_ID).getLibelle(),
											getDocumentModel().getDocumentId(),
											"etatPaye",
											"payeIndivMensuel", // lment importable
											refNodeId, // id lment importable
											localName,
											libelleLien,
											msg,
											getDocumentModel().getControlById(CTRL_ID).getRegle().getHtmlContent(),
											getDocumentModel().getControlById(CTRL_ID).getSeverity(),
											null, // xpath vers le bordereau sous lequel se trouve l'anomalie
											xslParams); // paramtres supplmentaires pour accder  l'lment en anomalie
				ano.addNode(new Node(refNodeId));
				anos.add(ano);
			} 
			if (!"".equals(codesDeclaresNonUtilises)) {
				msg = "Les codes vnements suivants sont dclars dans la nomenclature mais pas utiliss : "+codesDeclaresNonUtilises;
				String path = ""; // path est vide de faon  ne pas avoir de lien sous l'anomalie. 
				String anoId = IdGenerator.nextId();
				String libelleLien = "";
				Hashtable<String, Object> xslParams = new Hashtable<String, Object>();
				xslParams.put("elementId", "Paye");
				xslParams.put("browser-destination", "internal");
				xslParams.put("anoId", anoId);
				Anomalie ano = new Anomalie(anoId,
											getDocumentModel().getControlById(CTRL_ID).getId(),
											getDocumentModel().getControlById(CTRL_ID).getLibelle(),
											getDocumentModel().getDocumentId(),
											"etatPaye",
											"payeIndivMensuel", // lment importable
											refNodeId, // id lment importable
											localName,
											libelleLien,
											msg,
											getDocumentModel().getControlById(CTRL_ID).getRegle().getHtmlContent(),
											getDocumentModel().getControlById(CTRL_ID).getSeverity(),
											path, // xpath vers le bordereau sous lequel se trouve l'anomalie
											xslParams); // paramtres supplmentaires pour accder  l'lment en anomalie
				ano.addNode(new Node(refNodeId));
				anos.add(ano);
			}
			return anos;
		} else {
			return null;
		}
	}

	public void characters(char[] ch, int start, int length, String xpath)
			throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void endDocument() throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void endPrefixMapping(String prefix, String xpath)
			throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void ignorableWhitespace(char[] ch, int start, int length,
			String xpath) throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void processingInstruction(String target, String data, String xpath)
			throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void setDocumentLocator(Locator locator) {
		// TODO Raccord de mthode auto-gnr

	}

	public void setParameters(Hashtable<String, Object> params) {
		// TODO Raccord de mthode auto-gnr

	}

	public void skippedEntity(String name, String xpath) throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

	public void startPrefixMapping(String prefix, String uri, String xpath)
			throws SAXException {
		// TODO Raccord de mthode auto-gnr

	}

}
