/*
 * Copyright 
 *   2007 axYus - www.axyus.com
 *   2007 JP.Tessier - jean-philippe.tessier@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.cp.xemelios.controls.models;

import java.util.Vector;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;
import java.util.Hashtable;
import javax.xml.namespace.QName;

/**
 * Modelise un ensemble de documents
 * 
 * @author chm
 */
public class ControlsModel implements XmlMarshallable {
	public static final transient String TAG = "controls";
	public static final transient QName QN = new QName(TAG);
	private Vector<DocumentControlModel> documents;
	private Hashtable<String, DocumentControlModel> hDocuments;
	private MessageRapportModel message;

	public ControlsModel(QName tag) {
		super();
		documents = new Vector<DocumentControlModel>();
		hDocuments = new Hashtable<String, DocumentControlModel>();
	}

	public void addCharacterData(String cData) throws SAXException {
		throw new SAXException("pas de CDATA attendu");
	}
	public void addChild(XmlMarshallable child, QName tagName)
			throws SAXException {
		if (DocumentControlModel.QN.equals(tagName)) {
			DocumentControlModel dcm = (DocumentControlModel) child;
			if (hDocuments.containsKey(dcm.getDocumentId())) {
				DocumentControlModel old = hDocuments.get(dcm.getDocumentId());
				documents.remove(old);
				hDocuments.remove(old.getDocumentId());
			}
			documents.add(dcm);
			hDocuments.put(dcm.getDocumentId(), dcm);
		}
		else if (MessageRapportModel.QN.equals(tagName)) {
			message = (MessageRapportModel)child;
		} else
			throw new SAXException(tagName + ": tag non attendu");
	}
	public XmlMarshallable getAttributes(XmlAttributes attributes)
			throws SAXException {
		return this;
	}
	public void marshall(XmlOutputter output) {
		output.startTag(TAG);
		if (message!=null) message.marshall(output);
		for (DocumentControlModel dm : documents)
			dm.marshall(output);
		output.endTag(TAG);
	}
	public void validate() throws InvalidXmlDefinition {
		for (DocumentControlModel dm : documents)
			dm.validate();
	}
	public Vector<DocumentControlModel> getDocuments() {
		return documents;
	}
	public DocumentControlModel getDocumentById(String docId) {
		// for(Enumeration<DocumentControlModel>
		// enumer=documents.elements();enumer.hasMoreElements();) {
		// DocumentControlModel doc = enumer.nextElement();
		// if(doc.getDocumentById().equals(docId)) return doc;
		// }
		return hDocuments.get(docId);
	}
	@Override
	public ControlsModel clone() {
		ControlsModel dom = new ControlsModel(QN);
		for (DocumentControlModel dm : this.documents)
			try {
				dom.addChild(dm, DocumentControlModel.QN);
			} catch (Exception ex) {
			}
		try {if (this.message!=null) dom.addChild(this.message, MessageRapportModel.QN);} catch (Throwable t) {}
		return dom;
	}

	public XmlMarshallable getChildToModify(String uri, String localName, String qName, Attributes atts) {
		if (DocumentControlModel.QN.equals(qName)) {
			return hDocuments.get(atts.getValue("id"));
		} else if (MessageRapportModel.QN.equals(qName)) {
			return message;
		} else {
			return null;
		}
	}

	public QName getQName() {
		return QN;
	}

	public MessageRapportModel getMessage() {
		return message;
	}

	public void setMessage(MessageRapportModel message) {
		this.message = message;
	}
}